#!/usr/bin/env python

import micropython.cmd
import rsvp
import sys
import os
from glob import glob
import operator

libdirs = [
    os.path.join(os.path.split(__file__)[0], "lib"),
    "/usr/share/micropython/lib"
    ]

# Main program.

if __name__ == "__main__":
    args = sys.argv[1:]
    path = libdirs + sys.path[:]
    path.append("tests")

    # Process all tests.

    try:
        _f = args.index("-f")
        filenames = args[_f+1:]
    except ValueError:
        filenames = glob(os.path.join("tests", "*.py"))

    filenames.sort()

    results = []

    for filename in filenames:
        print "Processing", filename

        try:
            p = micropython.cmd.get_program(path, args)
            m = p.get_importer().load_from_file(filename)

        # Report any errors.

        except micropython.ProcessingError, exc:
            print repr(exc)
            if "-tb" in args:
                raise
            results.append((filename, 0))

        else:
            rm = rsvp.machine(p)
            success = rm.test(m)
            print "Test successful?", success and "Yes" or "No"
            results.append((filename, success))

    failed = [result[0] for result in results if not result[1]]
    if failed:
        print
        print "Failed tests:"
        for filename in failed:
            print filename

    print
    print "All successful?"
    print reduce(operator.and_, [x[1] for x in results], 1) and "Yes" or "No"

# vim: tabstop=4 expandtab shiftwidth=4
